@@ -11,11 +11,10 @@ from hypothesis.errors import InvalidArgument
 from hypothesis.strategies._internal.utils import cacheable
 
 from .. import nodes
-from ..types import AstPrinter, CustomScalars, Field, InputTypeNode, InterfaceOrObject, SelectionNodes
-from . import factories, primitives
+from ..types import AstPrinter, CustomScalarStrategies, Field, InputTypeNode, InterfaceOrObject, SelectionNodes
+from . import factories, primitives, validation
 from .ast import make_mutation, make_query
 from .containers import flatten
-from .validation import maybe_parse_schema, validate_custom_scalars, validate_fields
 
 BY_NAME = operator.attrgetter("name")
 EMPTY_LISTS_STRATEGY = st.builds(list)
@@ -45,7 +44,7 @@ class GraphQLStrategy:
     """Strategy for generating various GraphQL nodes."""
 
     schema: graphql.GraphQLSchema = attr.ib()
-    custom_scalars: CustomScalars = attr.ib(factory=dict)
+    custom_scalars: CustomScalarStrategies = attr.ib(factory=dict)
     # As the schema is assumed to be immutable, there are a few strategy caches possible for internal components
     # This is a per-method cache without limits as they are proportionate to the schema size
     _cache: Dict[str, Dict] = attr.ib(factory=dict)
@@ -318,6 +317,10 @@ def subset_of_fields(
     fields: Dict[str, graphql.GraphQLInputField], *, force_required: bool = False
 ) -> st.SearchStrategy[List[Tuple[str, graphql.GraphQLInputField]]]:
     """A helper to select a subset of fields."""
+    if not fields:
+        # The schema is invalid as there should be at least one field
+        # But there should not be an internal error because of it
+        return EMPTY_LISTS_STRATEGY
     field_pairs = sorted(fields.items())
     # if we need to always generate required fields, then return them and extend with a subset of optional fields
     if force_required:
@@ -341,13 +344,13 @@ def _make_strategy(
     *,
     type_: graphql.GraphQLObjectType,
     fields: Optional[Iterable[str]] = None,
-    custom_scalars: Optional[CustomScalars] = None,
+    custom_scalars: Optional[CustomScalarStrategies] = None,
 ) -> st.SearchStrategy[List[graphql.FieldNode]]:
     if fields is not None:
         fields = tuple(fields)
-        validate_fields(fields, type_.fields)
+        validation.validate_fields(fields, type_.fields)
     if custom_scalars:
-        validate_custom_scalars(custom_scalars)
+        validation.validate_custom_scalars(custom_scalars)
     return GraphQLStrategy(schema, custom_scalars or {}).selections(type_, fields=fields)
 
 
@@ -356,7 +359,7 @@ def queries(
     schema: Union[str, graphql.GraphQLSchema],
     *,
     fields: Optional[Iterable[str]] = None,
-    custom_scalars: Optional[CustomScalars] = None,
+    custom_scalars: Optional[CustomScalarStrategies] = None,
     print_ast: AstPrinter = graphql.print_ast,
 ) -> st.SearchStrategy[str]:
     """A strategy for generating valid queries for the given GraphQL schema.
@@ -368,7 +371,7 @@ def queries(
     :param custom_scalars: Strategies for generating custom scalars.
     :param print_ast: A function to convert the generated AST to a string.
     """
-    parsed_schema = maybe_parse_schema(schema)
+    parsed_schema = validation.maybe_parse_schema(schema)
     if parsed_schema.query_type is None:
         raise ValueError("Query type is not defined in the schema")
     return (
@@ -383,7 +386,7 @@ def mutations(
     schema: Union[str, graphql.GraphQLSchema],
     *,
     fields: Optional[Iterable[str]] = None,
-    custom_scalars: Optional[CustomScalars] = None,
+    custom_scalars: Optional[CustomScalarStrategies] = None,
     print_ast: AstPrinter = graphql.print_ast,
 ) -> st.SearchStrategy[str]:
     """A strategy for generating valid mutations for the given GraphQL schema.
@@ -395,7 +398,7 @@ def mutations(
     :param custom_scalars: Strategies for generating custom scalars.
     :param print_ast: A function to convert the generated AST to a string.
     """
-    parsed_schema = maybe_parse_schema(schema)
+    parsed_schema = validation.maybe_parse_schema(schema)
     if parsed_schema.mutation_type is None:
         raise ValueError("Mutation type is not defined in the schema")
     return (
